/*
** Module   : LargeObjectOps.java
** Abstract : helper routines to process LOBs
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 GES 20130516 First version implementing an early version of the COPY-LOB statement which
**                  is suitable for executing testcases but is not yet 100% complete.
*/

package com.goldencode.p2j.util;

import java.io.*;

/**
 * Provides static utility methods for processing large objects (LOBs) in a Progress 4GL
 * compatible manner.
 * <p> 
 * At this time, the following Progress 4GL features are handled:
 * <p>
 * <pre>
 * Progress                       Java Equivalent
 * ---------------------------    ---------------------------------------------------------
 * COPY-LOB                       writeToFile
 * </pre>
 */
public class LargeObjectOps
{
   /**
    * Private constructor to ensure instances are not created. All functionality of this class
    * is to be used via static methods.
    */
   private LargeObjectOps()
   {
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    */
   public static void writeToFile(BinaryData data, character fname)
   {
      writeToFile(data, fname, false);
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    * @param    append
    *           <code>true</code> if the contents should be appended to the file.
    */
   public static void writeToFile(BinaryData data, character fname, boolean append)
   {
      if (fname == null || fname.isUnknown())
      {
         return;
      }
      
      writeToFile(data, fname.toStringMessage(), append);
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    */
   public static void writeToFile(BinaryData data, String fname)
   {
      writeToFile(data, fname, false);
   }
   
   /**
    * Write the full contents of the given binary object to the file specified by the given
    * filename.
    *
    * @param    data
    *           The data to write.
    * @param    fname
    *           The target filename.
    * @param    append
    *           <code>true</code> if the contents should be appended to the file.
    */
   public static void writeToFile(BinaryData data, String fname, boolean append)
   {
      // TODO: this is just a first pass, the real/full behavior needs to be determined
      
      if (data == null || data.isUnknown() || fname == null || fname.length() == 0)
      {
         return;
      }
      
      Stream file     = StreamFactory.openFileStream(fname, true, append);
      byte[] contents = data.getByteArray();
      
      try
      {
         // empty array should yield an empty file
         if (contents.length > 0)
         {
            file.write(contents);
         }
      }
      
      catch (InterruptedIOException iioe)
      {
         throw new StopConditionException(iioe);
      }
      
      catch (IOException ioe)
      {
         ErrorManager.showErrorAndAbend(140, Stream.PIPE_BROKEN_MESSAGE);
      }
      
      finally
      {
         file.close();
      }
   }
}
