/*
** Module   : memory.c
** Abstract : implementation of low level memptr functions called via JNI
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 GES 20130429 First version. This code should work on all operating systems and hardware
**                  architectures. The idea is to provide the minimum set of low level memory
**                  functions to enable the full set of memptr features to be implemented in
**                  pure Java. This class has no error processing or boundary checking. That
**                  is expected to be done in the caller.
*/ 

#include <stdlib.h>
#include <malloc.h>

#include "com_goldencode_p2j_util_MemoryManager.h"
                                           
/**
 * Reports on the native byte-ordering of the platform on which this library is
 * running. 
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 *
 * @return   true if the platform uses little-endian byte-ordering, false if the platform
 *           is big-endian.
 */ 
JNIEXPORT jboolean JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_isLittleEndian(JNIEnv* env, jclass cls)
{
   // allocate a 32-bit integer on the stack and set its value such that only the most
   // significant byte is 0x00
   jint num = 0x00FFFFFF;
   
   // obtain the pointer to the lowest addressable byte in the integer
   char* base = (char*) &num;

   // if the contents of the lowest byte are non-zero, then it must not contain the most
   // significant byte and thus the platform is little-endian
   return *base ? JNI_TRUE : JNI_FALSE;   
}

/**
 * Allocate the given number of bytes as a new memory block and return the 64-bit address. All
 * bytes in the memory region will be initialized to 0x00.
 * <p>
 * To duplicate the permissiveness of the 4GL memptr allocation (where certain operations often
 * can read or write past the end of the allocated buffer without an access violation), the
 * buffer allocated by this function will always be 16 bytes larger than the requested size. 
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 * @param    len
 *           The size in bytes of the memory region to allocate.
 *
 * @return   The address of the allocated memory region or 0 if memory could not be allocated.
 */ 
JNIEXPORT jlong JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_allocate(JNIEnv* env, jclass cls, jlong len)
{
   return (jlong) calloc(len, sizeof(jbyte) + 16);   
}

/**
 * De-allocate the memory block at the given 64-bit address.  Memory in this region MUST NOT
 * be accessed after this call. 
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 * @param    addr
 *           The 64-bit pointer to a memory region previously allocated using this module.
 *           DO NOT pass in an invalid pointer.  DO NOT deallocate the same pointer more than
 *           once.
 */ 
JNIEXPORT void JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_deallocate(JNIEnv* env, jclass cls, jlong addr)
{
   free((void*) addr);
}

/**
 * Returns the next index position into the array which is the <code>null</code> byte, starting
 * at the given index position.
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 * @param    addr
 *           The 64-bit pointer to a memory region previously allocated using this module.
 *           DO NOT pass in an invalid pointer.
 * @param    idx
 *           The index position at which to start the search.
 * @param    size
 *           The length of the buffer. If the search reaches this byte without finding a
 *           null byte, then this value will be returned. If this is given as -1, then the
 *           caller has no size information and we operate in unsafe mode.  In this mode
 *           the code will read until it finds a <code>null</code> or gets an access
 *           violation.  This is how the 4GL does it too.
 *                                                                        
 * @return   The index of the next byte position which contains the <code>null</code> byte or
 *           the length of the array if no <code>null</code> byte is contained.
 */ 
JNIEXPORT jlong JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_findNextNull(JNIEnv* env,
                                                           jclass  cls,
                                                           jlong   addr,
                                                           jlong   idx,
                                                           jlong   size)
{
   char* value = (char*) addr;
   jlong i     = idx; 
   
   // not all instances will end in null byte, so check length first to
   // prevent overrun
   while ((size == -1 || i < size) && value[i] != '\0')
   {
      i++;
   }
   
   return i;
}

/**                                                       
 * Read the specified range of contents from the memory block at the given 64-bit address.
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 * @param    addr
 *           The 64-bit pointer to a memory region previously allocated using this module.
 *           DO NOT pass in an invalid pointer.
 * @param    offset
 *           The 0-based index position from the beginning of the buffer, from which to start
 *           reading.
 * @param    len
 *           The number of bytes to read. Must be greater than 0 and must NOT extend past the
 *           end of the buffer.
 *
 * @return   The range of bytes read or NULL if there is any problem allocating the array.
 */ 
JNIEXPORT jbyteArray JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_read(JNIEnv* env,
                                                   jclass  cls,
                                                   jlong   addr,
                                                   jlong   offset,
                                                   jlong   len)
{
   jbyteArray data = (*env)->NewByteArray(env, len);
   
   if (data != NULL)
   {
      (*env)->SetByteArrayRegion(env, data, 0, (jsize) len, (jbyte*) addr + offset); 
   }
   
   return data;
}

/**
 * Write the specified contents to the memory block at the given 64-bit address plus offset.
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 * @param    addr
 *           The 64-bit pointer to a memory region previously allocated using this module.
 *           DO NOT pass in an invalid pointer.
 * @param    offset
 *           The 0-based index position from the beginning of the buffer, at which to start
 *           writing.
 * @param    data
 *           The bytes to write.  The entire contents of the array will be written. The length
 *           of the array must not be bigger than the number of bytes between the offset and
 *           the end of the buffer.
 *
 * @return   The range of bytes read.
 */ 
JNIEXPORT void JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_write(JNIEnv*    env,
                                                    jclass     cls,
                                                    jlong      addr,
                                                    jlong      offset,
                                                    jbyteArray data)
{
   jsize len = (*env)->GetArrayLength(env, data);
   (*env)->GetByteArrayRegion(env, data, 0, len, (jbyte*) addr + offset);
}

/**
 * Copy the specified range of contents from the source memory block to the destination memory
 * block.
 *
 * @param    env
 *           Pointer to a structure that provides access to JNI environment.
 * @param    cls
 *           Reference to the Java class which called this function (from static context).
 * @param    source
 *           The 64-bit pointer to a memory region previously allocated using this module.
 *           DO NOT pass in an invalid pointer.
 * @param    dest
 *           The 64-bit pointer to a memory region previously allocated using this module.
 *           DO NOT pass in an invalid pointer.
 * @param    offset
 *           The 0-based index position from the beginning of the buffer, at which to start
 *           writing.
 * @param    len
 *           The number of bytes to write. Must be greater than 0 and must NOT extend past the
 *           end of the destination buffer.
 */ 
JNIEXPORT void JNICALL
   Java_com_goldencode_p2j_util_MemoryManager_copy(JNIEnv* env,
                                                   jclass  cls,
                                                   jlong   source,
                                                   jlong   dest,
                                                   jlong   offset,
                                                   jlong   len)
{                                              
   jbyte* src = (jbyte*) source;
   jbyte* dst = (jbyte*) (dest + offset);
   
   jlong i;
   
   for (i = 0; i < len; i++)
   {
      dst[i] = src[i];
   }
}
